# -*- coding: utf-8 -*-
# Authored by: Ryan

from amazon_ad.api.base import ZADOpenAPI

ALLOW_REPORT_TYPES = [
    "campaigns",
    "adGroups",
    "productAds",
    "targets",
    "asins",
]

# 所有的广告策略
ALL_TACTICS = ['T00001', 'T00020', 'T00030', 'remarketing']

REPORT_TYPE_METRIC_MAPPING = {
    'campaigns': {
        'T00020': ['campaignName',
                   'campaignId',

                   'campaignStatus',  # new
                   'campaignBudget',  # new
                   'attributedDetailPageView14d',  # new
                   'viewAttributedDetailPageView14d',  # new
                   
                   'impressions',
                   'clicks',
                   'cost',
                   'currency',
                   'attributedConversions1d',
                   'attributedConversions7d',
                   'attributedConversions14d',
                   'attributedConversions30d',
                   'attributedConversions1dSameSKU',
                   'attributedConversions7dSameSKU',
                   'attributedConversions14dSameSKU',
                   'attributedConversions30dSameSKU',
                   'attributedUnitsOrdered1d',
                   'attributedUnitsOrdered7d',
                   'attributedUnitsOrdered14d',
                   'attributedUnitsOrdered30d',
                   'attributedSales1d',
                   'attributedSales7d',
                   'attributedSales14d',
                   'attributedSales30d',
                   'attributedSales1dSameSKU',
                   'attributedSales7dSameSKU',
                   'attributedSales14dSameSKU',
                   'attributedSales30dSameSKU',
                   'attributedOrdersNewToBrand14d',
                   'attributedSalesNewToBrand14d',
                   'attributedUnitsOrderedNewToBrand14d',
                   'costType',
                   'viewImpressions',
                   'viewAttributedConversions14d',
                   'viewAttributedSales14d',
                   'viewAttributedUnitsOrdered14d'],
        'T00030': ['campaignName',
                   'campaignId',

                   'campaignStatus',  # new
                   'campaignBudget',  # new
                   'attributedDetailPageView14d',  # new
                   'viewAttributedDetailPageView14d',  # new

                   'impressions',
                   'clicks',
                   'cost',
                   'currency',
                   'attributedConversions1d',
                   'attributedConversions7d',
                   'attributedConversions14d',
                   'attributedConversions30d',
                   'attributedConversions1dSameSKU',
                   'attributedConversions7dSameSKU',
                   'attributedConversions14dSameSKU',
                   'attributedConversions30dSameSKU',
                   'attributedUnitsOrdered1d',
                   'attributedUnitsOrdered7d',
                   'attributedUnitsOrdered14d',
                   'attributedUnitsOrdered30d',
                   'attributedSales1d',
                   'attributedSales7d',
                   'attributedSales14d',
                   'attributedSales30d',
                   'attributedSales1dSameSKU',
                   'attributedSales7dSameSKU',
                   'attributedSales14dSameSKU',
                   'attributedSales30dSameSKU',
                   'attributedOrdersNewToBrand14d',
                   'attributedSalesNewToBrand14d',
                   'attributedUnitsOrderedNewToBrand14d',
                   'costType',
                   'viewImpressions',
                   'viewAttributedConversions14d',
                   'viewAttributedSales14d',
                   'viewAttributedUnitsOrdered14d'],
        'remarketing': ['campaignName',
                        'campaignId',
                        'impressions',
                        'clicks',
                        'cost',
                        'currency',
                        'attributedConversions1d',
                        'attributedConversions7d',
                        'attributedConversions14d',
                        'attributedConversions30d',
                        'attributedConversions1dSameSKU',
                        'attributedConversions7dSameSKU',
                        'attributedConversions14dSameSKU',
                        'attributedConversions30dSameSKU',
                        'attributedUnitsOrdered1d',
                        'attributedUnitsOrdered7d',
                        'attributedUnitsOrdered14d',
                        'attributedUnitsOrdered30d',
                        'attributedSales1d',
                        'attributedSales7d',
                        'attributedSales14d',
                        'attributedSales30d',
                        'attributedSales1dSameSKU',
                        'attributedSales7dSameSKU',
                        'attributedSales14dSameSKU',
                        'attributedSales30dSameSKU'],
        'T00001': ['campaignName',
                   'campaignId',
                   'campaignStatus',
                   'currency',
                   'impressions',
                   'clicks',
                   'cost',
                   'attributedDPV14d',
                   'attributedUnitsSold14d',
                   'attributedSales14d']},
    'adGroups': {'T00020': ['campaignName',
                            'campaignId',
                            'adGroupName',
                            'adGroupId',

                            'attributedDetailPageView14d',  # new
                            'viewAttributedDetailPageView14d',  # new

                            'impressions',
                            'clicks',
                            'cost',
                            'currency',
                            'attributedConversions1d',
                            'attributedConversions7d',
                            'attributedConversions14d',
                            'attributedConversions30d',
                            'attributedConversions1dSameSKU',
                            'attributedConversions7dSameSKU',
                            'attributedConversions14dSameSKU',
                            'attributedConversions30dSameSKU',
                            'attributedUnitsOrdered1d',
                            'attributedUnitsOrdered7d',
                            'attributedUnitsOrdered14d',
                            'attributedUnitsOrdered30d',
                            'attributedSales1d',
                            'attributedSales7d',
                            'attributedSales14d',
                            'attributedSales30d',
                            'attributedSales1dSameSKU',
                            'attributedSales7dSameSKU',
                            'attributedSales14dSameSKU',
                            'attributedSales30dSameSKU',
                            'attributedOrdersNewToBrand14d',
                            'attributedSalesNewToBrand14d',
                            'attributedUnitsOrderedNewToBrand14d',
                            'bidOptimization',
                            'viewImpressions',
                            'viewAttributedConversions14d',
                            'viewAttributedSales14d',
                            'viewAttributedUnitsOrdered14d'],
                 'T00030': ['campaignName',
                            'campaignId',
                            'adGroupName',
                            'adGroupId',

                            'attributedDetailPageView14d',  # new
                            'viewAttributedDetailPageView14d',  # new

                            'impressions',
                            'clicks',
                            'cost',
                            'currency',
                            'attributedConversions1d',
                            'attributedConversions7d',
                            'attributedConversions14d',
                            'attributedConversions30d',
                            'attributedConversions1dSameSKU',
                            'attributedConversions7dSameSKU',
                            'attributedConversions14dSameSKU',
                            'attributedConversions30dSameSKU',
                            'attributedUnitsOrdered1d',
                            'attributedUnitsOrdered7d',
                            'attributedUnitsOrdered14d',
                            'attributedUnitsOrdered30d',
                            'attributedSales1d',
                            'attributedSales7d',
                            'attributedSales14d',
                            'attributedSales30d',
                            'attributedSales1dSameSKU',
                            'attributedSales7dSameSKU',
                            'attributedSales14dSameSKU',
                            'attributedSales30dSameSKU',
                            'attributedOrdersNewToBrand14d',
                            'attributedSalesNewToBrand14d',
                            'attributedUnitsOrderedNewToBrand14d',
                            'bidOptimization',
                            'viewImpressions',
                            'viewAttributedConversions14d',
                            'viewAttributedSales14d',
                            'viewAttributedUnitsOrdered14d'],
                 'remarketing': ['campaignName',
                                 'campaignId',
                                 'adGroupName',
                                 'adGroupId',
                                 'impressions',
                                 'clicks',
                                 'cost',
                                 'currency',
                                 'attributedConversions1d',
                                 'attributedConversions7d',
                                 'attributedConversions14d',
                                 'attributedConversions30d',
                                 'attributedConversions1dSameSKU',
                                 'attributedConversions7dSameSKU',
                                 'attributedConversions14dSameSKU',
                                 'attributedConversions30dSameSKU',
                                 'attributedUnitsOrdered1d',
                                 'attributedUnitsOrdered7d',
                                 'attributedUnitsOrdered14d',
                                 'attributedUnitsOrdered30d',
                                 'attributedSales1d',
                                 'attributedSales7d',
                                 'attributedSales14d',
                                 'attributedSales30d',
                                 'attributedSales1dSameSKU',
                                 'attributedSales7dSameSKU',
                                 'attributedSales14dSameSKU',
                                 'attributedSales30dSameSKU']},
    'productAds': {'T00020': ['campaignName',
                              'campaignId',
                              'adGroupName',
                              'adGroupId',

                              'attributedDetailPageView14d',  # new
                              'viewAttributedDetailPageView14d',  # new

                              'asin',
                              'sku',
                              'adId',
                              'impressions',
                              'clicks',
                              'cost',
                              'currency',
                              'attributedConversions1d',
                              'attributedConversions7d',
                              'attributedConversions14d',
                              'attributedConversions30d',
                              'attributedConversions1dSameSKU',
                              'attributedConversions7dSameSKU',
                              'attributedConversions14dSameSKU',
                              'attributedConversions30dSameSKU',
                              'attributedUnitsOrdered1d',
                              'attributedUnitsOrdered7d',
                              'attributedUnitsOrdered14d',
                              'attributedUnitsOrdered30d',
                              'attributedSales1d',
                              'attributedSales7d',
                              'attributedSales14d',
                              'attributedSales30d',
                              'attributedSales1dSameSKU',
                              'attributedSales7dSameSKU',
                              'attributedSales14dSameSKU',
                              'attributedSales30dSameSKU',
                              'attributedOrdersNewToBrand14d',
                              'attributedSalesNewToBrand14d',
                              'attributedUnitsOrderedNewToBrand14d',
                              'viewImpressions',
                              'viewAttributedConversions14d',
                              'viewAttributedSales14d',
                              'viewAttributedUnitsOrdered14d'],
                   'T00030': ['campaignName',
                              'campaignId',
                              'adGroupName',
                              'adGroupId',

                              'attributedDetailPageView14d',  # new
                              'viewAttributedDetailPageView14d',  # new

                              'asin',
                              'sku',
                              'adId',
                              'impressions',
                              'clicks',
                              'cost',
                              'currency',
                              'attributedConversions1d',
                              'attributedConversions7d',
                              'attributedConversions14d',
                              'attributedConversions30d',
                              'attributedConversions1dSameSKU',
                              'attributedConversions7dSameSKU',
                              'attributedConversions14dSameSKU',
                              'attributedConversions30dSameSKU',
                              'attributedUnitsOrdered1d',
                              'attributedUnitsOrdered7d',
                              'attributedUnitsOrdered14d',
                              'attributedUnitsOrdered30d',
                              'attributedSales1d',
                              'attributedSales7d',
                              'attributedSales14d',
                              'attributedSales30d',
                              'attributedSales1dSameSKU',
                              'attributedSales7dSameSKU',
                              'attributedSales14dSameSKU',
                              'attributedSales30dSameSKU',
                              'attributedOrdersNewToBrand14d',
                              'attributedSalesNewToBrand14d',
                              'attributedUnitsOrderedNewToBrand14d',
                              'viewImpressions',
                              'viewAttributedConversions14d',
                              'viewAttributedSales14d',
                              'viewAttributedUnitsOrdered14d'],
                   'remarketing': ['campaignName',
                                   'campaignId',
                                   'adGroupName',
                                   'adGroupId',
                                   'asin',
                                   'sku',
                                   'adId',
                                   'impressions',
                                   'clicks',
                                   'cost',
                                   'currency',
                                   'attributedConversions1d',
                                   'attributedConversions7d',
                                   'attributedConversions14d',
                                   'attributedConversions30d',
                                   'attributedConversions1dSameSKU',
                                   'attributedConversions7dSameSKU',
                                   'attributedConversions14dSameSKU',
                                   'attributedConversions30dSameSKU',
                                   'attributedUnitsOrdered1d',
                                   'attributedUnitsOrdered7d',
                                   'attributedUnitsOrdered14d',
                                   'attributedUnitsOrdered30d',
                                   'attributedSales1d',
                                   'attributedSales7d',
                                   'attributedSales14d',
                                   'attributedSales30d',
                                   'attributedSales1dSameSKU',
                                   'attributedSales7dSameSKU',
                                   'attributedSales14dSameSKU',
                                   'attributedSales30dSameSKU']},
    'targets': {'T00020': ['campaignName',
                           'campaignId',
                           'adGroupName',
                           'adGroupId',
                           'targetId',
                           'targetingExpression',
                           'targetingText',
                           'targetingType',

                           'attributedDetailPageView14d',  # new
                           'viewAttributedDetailPageView14d',  # new

                           'impressions',
                           'clicks',
                           'cost',
                           'currency',
                           'attributedConversions1d',
                           'attributedConversions7d',
                           'attributedConversions14d',
                           'attributedConversions30d',
                           'attributedConversions1dSameSKU',
                           'attributedConversions7dSameSKU',
                           'attributedConversions14dSameSKU',
                           'attributedConversions30dSameSKU',
                           'attributedUnitsOrdered1d',
                           'attributedUnitsOrdered7d',
                           'attributedUnitsOrdered14d',
                           'attributedUnitsOrdered30d',
                           'attributedSales1d',
                           'attributedSales7d',
                           'attributedSales14d',
                           'attributedSales30d',
                           'attributedSales1dSameSKU',
                           'attributedSales7dSameSKU',
                           'attributedSales14dSameSKU',
                           'attributedSales30dSameSKU',
                           'attributedOrdersNewToBrand14d',
                           'attributedSalesNewToBrand14d',
                           'attributedUnitsOrderedNewToBrand14d',
                           'viewImpressions',
                           'viewAttributedConversions14d',
                           'viewAttributedSales14d',
                           'viewAttributedUnitsOrdered14d'],
                'T00030': ['campaignName',
                           'campaignId',
                           'adGroupName',
                           'adGroupId',
                           'targetId',
                           'targetingExpression',
                           'targetingText',
                           'targetingType',

                           'attributedDetailPageView14d',  # new
                           'viewAttributedDetailPageView14d',  # new

                           'impressions',
                           'clicks',
                           'cost',
                           'currency',
                           'attributedConversions1d',
                           'attributedConversions7d',
                           'attributedConversions14d',
                           'attributedConversions30d',
                           'attributedConversions1dSameSKU',
                           'attributedConversions7dSameSKU',
                           'attributedConversions14dSameSKU',
                           'attributedConversions30dSameSKU',
                           'attributedUnitsOrdered1d',
                           'attributedUnitsOrdered7d',
                           'attributedUnitsOrdered14d',
                           'attributedUnitsOrdered30d',
                           'attributedSales1d',
                           'attributedSales7d',
                           'attributedSales14d',
                           'attributedSales30d',
                           'attributedSales1dSameSKU',
                           'attributedSales7dSameSKU',
                           'attributedSales14dSameSKU',
                           'attributedSales30dSameSKU',
                           'attributedOrdersNewToBrand14d',
                           'attributedSalesNewToBrand14d',
                           'attributedUnitsOrderedNewToBrand14d',
                           'viewImpressions',
                           'viewAttributedConversions14d',
                           'viewAttributedSales14d',
                           'viewAttributedUnitsOrdered14d']},
    'asins': {'T00020': ['campaignName',
                         'campaignId',
                         'adGroupName',
                         'adGroupId',

                         'attributedDetailPageView14d',  # new
                         'viewAttributedDetailPageView14d',  # new
                         'costType',  # new
                         'attributedOrdersNewToBrand14d',  # new
                         'attributedSalesNewToBrand14d',  # new
                         'attributedUnitsOrderedNewToBrand14d',  # new

                         'asin',
                         'otherAsin',
                         'sku',
                         'currency',
                         'attributedUnitsOrdered1dOtherSKU',
                         'attributedUnitsOrdered7dOtherSKU',
                         'attributedUnitsOrdered14dOtherSKU',
                         'attributedUnitsOrdered30dOtherSKU',
                         'attributedSales1dOtherSKU',
                         'attributedSales7dOtherSKU',
                         'attributedSales14dOtherSKU',
                         'attributedSales30dOtherSKU'],
              'T00030': ['campaignName',
                         'campaignId',
                         'adGroupName',
                         'adGroupId',

                         'attributedDetailPageView14d',  # new
                         'viewAttributedDetailPageView14d',  # new
                         'costType',  # new
                         'attributedOrdersNewToBrand14d',  # new
                         'attributedSalesNewToBrand14d',  # new
                         'attributedUnitsOrderedNewToBrand14d',  # new

                         'asin',
                         'otherAsin',
                         'sku',
                         'currency',
                         'attributedUnitsOrdered1dOtherSKU',
                         'attributedUnitsOrdered7dOtherSKU',
                         'attributedUnitsOrdered14dOtherSKU',
                         'attributedUnitsOrdered30dOtherSKU',
                         'attributedSales1dOtherSKU',
                         'attributedSales7dOtherSKU',
                         'attributedSales14dOtherSKU',
                         'attributedSales30dOtherSKU']}}


class SdReport(ZADOpenAPI):
    def request(self, record_type, report_date, tactic, metrics):
        """
        POST    /sd/{recordType}/report
        """

        path = '/sd/{record_type}/report'.format(record_type=record_type)

        if isinstance(metrics, (list, tuple)):
            metrics = ','.join(metrics)

        data = {
            'reportDate': report_date,
            'tactic': tactic,
            'metrics': metrics
        }
        return self.post(path, data)

    def _get_metrics(self, metrics, default):
        if not metrics:
            metrics = default

        return metrics

    def campaigns(self, report_date, tactic, metrics=None, **kwargs):

        metrics = self._get_metrics(metrics, REPORT_TYPE_METRIC_MAPPING.get('campaigns').get(tactic))

        return self.request('campaigns', report_date, tactic, metrics)

    def campaigns_t00001(self, report_date, metrics=None, **kwargs):
        tactic = 'T00001'
        metrics = self._get_metrics(metrics, REPORT_TYPE_METRIC_MAPPING.get('campaigns').get(tactic))
        return self.request('campaigns', report_date, tactic, metrics)

    def campaigns_t00020(self, report_date, metrics=None, **kwargs):
        tactic = 'T00020'
        metrics = self._get_metrics(metrics, REPORT_TYPE_METRIC_MAPPING.get('campaigns').get(tactic))
        return self.request('campaigns', report_date, tactic, metrics)

    def campaigns_t00030(self, report_date, metrics=None, **kwargs):
        tactic = 'T00030'
        metrics = self._get_metrics(metrics, REPORT_TYPE_METRIC_MAPPING.get('campaigns').get(tactic))
        return self.request('campaigns', report_date, tactic, metrics)

    def campaigns_remarketing(self, report_date, metrics=None, **kwargs):
        tactic = 'remarketing'
        metrics = self._get_metrics(metrics, REPORT_TYPE_METRIC_MAPPING.get('campaigns').get(tactic))
        return self.request('campaigns', report_date, tactic, metrics)

    def ad_groups(self, report_date, tactic, metrics=None, **kwargs):

        metrics = self._get_metrics(metrics, REPORT_TYPE_METRIC_MAPPING.get('adGroups').get(tactic))

        return self.request('adGroups', report_date, tactic, metrics)

    def ad_groups_t00020(self, report_date, metrics=None, **kwargs):
        tactic = 'T00020'
        metrics = self._get_metrics(metrics, REPORT_TYPE_METRIC_MAPPING.get('adGroups').get(tactic))
        return self.request('adGroups', report_date, tactic, metrics)

    def ad_groups_t00030(self, report_date, metrics=None, **kwargs):
        tactic = 'T00030'
        metrics = self._get_metrics(metrics, REPORT_TYPE_METRIC_MAPPING.get('adGroups').get(tactic))
        return self.request('adGroups', report_date, tactic, metrics)

    def ad_groups_remarketing(self, report_date, metrics=None, **kwargs):
        tactic = 'remarketing'
        metrics = self._get_metrics(metrics, REPORT_TYPE_METRIC_MAPPING.get('adGroups').get(tactic))
        return self.request('adGroups', report_date, tactic, metrics)

    def product_ads(self, report_date, tactic, metrics=None, **kwargs):

        metrics = self._get_metrics(metrics, REPORT_TYPE_METRIC_MAPPING.get('productAds').get(tactic))

        return self.request('productAds', report_date, tactic, metrics)

    def product_ads_t00020(self, report_date, metrics=None, **kwargs):
        tactic = 'T00020'
        metrics = self._get_metrics(metrics, REPORT_TYPE_METRIC_MAPPING.get('productAds').get(tactic))
        return self.request('productAds', report_date, tactic, metrics)

    def product_ads_t00030(self, report_date, metrics=None, **kwargs):
        tactic = 'T00030'
        metrics = self._get_metrics(metrics, REPORT_TYPE_METRIC_MAPPING.get('productAds').get(tactic))
        return self.request('productAds', report_date, tactic, metrics)

    def product_ads_remarketing(self, report_date, metrics=None, **kwargs):
        tactic = 'remarketing'
        metrics = self._get_metrics(metrics, REPORT_TYPE_METRIC_MAPPING.get('productAds').get(tactic))
        return self.request('productAds', report_date, tactic, metrics)

    def targets(self, report_date, tactic, metrics=None, **kwargs):

        metrics = self._get_metrics(metrics, REPORT_TYPE_METRIC_MAPPING.get('targets').get(tactic))

        return self.request('targets', report_date, tactic, metrics)

    def targets_t00020(self, report_date, metrics=None, **kwargs):
        tactic = 'T00020'
        metrics = self._get_metrics(metrics, REPORT_TYPE_METRIC_MAPPING.get('targets').get(tactic))
        return self.request('targets', report_date, tactic, metrics)

    def targets_t00030(self, report_date, metrics=None, **kwargs):
        tactic = 'T00030'
        metrics = self._get_metrics(metrics, REPORT_TYPE_METRIC_MAPPING.get('targets').get(tactic))
        return self.request('targets', report_date, tactic, metrics)

    def asins(self, report_date, tactic, metrics=None, **kwargs):

        metrics = self._get_metrics(metrics, REPORT_TYPE_METRIC_MAPPING.get('asins').get(tactic))

        return self.request('asins', report_date, tactic, metrics)

    def asins_t00020(self, report_date, metrics=None, **kwargs):
        tactic = 'T00020'
        metrics = self._get_metrics(metrics, REPORT_TYPE_METRIC_MAPPING.get('asins').get(tactic))
        return self.request('asins', report_date, tactic, metrics)

    def asins_t00030(self, report_date, metrics=None, **kwargs):
        tactic = 'T00030'
        metrics = self._get_metrics(metrics, REPORT_TYPE_METRIC_MAPPING.get('asins').get(tactic))
        return self.request('asins', report_date, tactic, metrics)

    def local_test(self, **kwargs):
        """
        测试用的接口
        """
        path = kwargs.get('path')
        data = kwargs.get('data')
        return self.post(path, data)



# -*- coding: utf-8 -*-
# Authored by: Josh (joshzda@gmail.com)
# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals
import inspect
import json
import logging
from urllib.parse import urljoin

import requests

from amazon_ad.core import exceptions
from amazon_ad.api.base import ZADBaseAPI
from amazon_ad.core.utils.serialize import json_loads

logger = logging.getLogger('zad')


class BaseClient(object):
    _http = requests.Session()

    API_BASE_URL = None

    def __new__(cls, *args, **kwargs):
        self = super(BaseClient, cls).__new__(cls)
        api_endpoints = inspect.getmembers(self, cls._is_api_endpoint)
        for name, api in api_endpoints:
            api_cls = type(api)
            api = api_cls(self)
            setattr(self, name, api)
        return self

    def __init__(self, prepare_mode=False, timeout=None, auto_retry=True, *args, **kwargs):
        self.prepare_mode = prepare_mode
        self.timeout = timeout
        self.auto_retry = auto_retry
        self._http = requests.Session()

    def get_api_base_url(self):
        return self.API_BASE_URL

    @staticmethod
    def _is_api_endpoint(obj):
        return isinstance(obj, ZADBaseAPI)

    def _prepare_request(self, method, url_or_endpoint, kwargs):
        api_base_url = kwargs.pop('api_base_url', self.get_api_base_url())
        if not url_or_endpoint.startswith(('http://', 'https://')):
            url = urljoin(api_base_url, url_or_endpoint)
        else:
            url = url_or_endpoint

        if 'params' not in kwargs:
            kwargs['params'] = {}
        if isinstance(kwargs.get('data', ''), dict):
            body = json.dumps(kwargs['data'], ensure_ascii=False)
            body = body.encode('utf-8')
            kwargs['data'] = body
            if 'headers' not in kwargs:
                kwargs['headers'] = {}
            kwargs['headers']['Content-Type'] = 'application/json'

        kwargs['timeout'] = kwargs.get('timeout', self.timeout)
        response_processor = kwargs.pop('response_processor', None)
        result_processor = kwargs.pop('result_processor', None)
        top_response_key = kwargs.pop('top_response_key', None)

        return method, url, kwargs, response_processor, result_processor, top_response_key

    def _request(self, method, url_or_endpoint, **kwargs):
        method, url, kwargs, response_processor, result_processor, top_response_key = self._prepare_request(method,
                                                                                                            url_or_endpoint,
                                                                                                            kwargs)
        res = self._http.request(
            method=method,
            url=url,
            **kwargs
        )

        self._check_request_response(res, url, method, **kwargs)

        result = self._handle_result(
            res, method, url, result_processor, top_response_key, response_processor, **kwargs
        )

        logger.debug(u"\n【请求地址】: %s\n【请求方法】: %s\n【请求参数】：%s \n%s\n【响应数据】：%s",
                     url, method, kwargs.get('params', ''), kwargs.get('data', ''), result)
        return result

    def _check_request_response(self, response, url, method, **kwargs):
        try:
            response.raise_for_status()
        except requests.RequestException as req_ex:
            self._log_request_error(req_ex, url, method, **kwargs)
            raise exceptions.ZADClientException(
                message=None,
                client=self,
                request=req_ex.request,
                response=req_ex.response
            )

    def _log_request_error(self, req_ex, url, method, **kwargs):
        logger.error("\n【请求地址】: %s\n【请求方法】: %s\n【请求参数】：%s \n%s\n【异常信息】：%s",
                     url, method, kwargs.get('params', ''), kwargs.get('data', ''), req_ex)

    def _decode_result(self, res):
        try:
            result = json_loads(res.content.decode('utf-8', 'ignore'), strict=False)
        except (TypeError, ValueError):
            # Return origin response object if we can not decode it as JSON
            logger.debug('Can not decode response as JSON', exc_info=True)
            return res
        return result

    def _top_result_processor(self, res, result, **kwargs):
        # 在此可以做接口业务的结果判断处理
        # if 'error_response' in result:
        #     error_response = result['error_response']
        #     logger.error("\n【请求地址】: %s\n【请求方法】: %s\n【请求参数】：%s \n%s\n【错误信息】：%s",
        #                  res.url, res.request.method, kwargs.get('params', ''), kwargs.get('data', ''), result)
        #     raise exceptions.ZADClientException(
        #         error_response.get('code', -1),
        #         error_response.get('sub_msg', ''),
        #         client=self,
        #         request=res.request,
        #         response=res
        #     )
        return result

    def _handle_result(self, res, method=None, url=None, result_processor=None, top_result_processor=None,
                       response_processor=None, **kwargs):

        if response_processor:
            return response_processor(response=res, client=self, **kwargs)

        if not isinstance(res, dict):
            result = self._decode_result(res)
        else:
            result = res

        if not isinstance(result, dict):
            return result

        if top_result_processor:
            result = top_result_processor(res, result, **kwargs)
        else:
            result = self._top_result_processor(res, result, **kwargs)

        return result if not result_processor else result_processor(result, response=res, client=self, **kwargs)

    def _handle_pre_request(self, method, uri, kwargs):
        return method, uri, kwargs

    def _handle_request_except(self, e, func, *args, **kwargs):
        raise e

    def request(self, method, uri, **kwargs):
        method, uri, kwargs = self._handle_pre_request(method, uri, kwargs)
        if self.prepare_mode:
            method, url, kwargs, response_processor, result_processor, top_response_key = self._prepare_request(method, uri, kwargs)
            return {
                'method':method,
                'url':url,
                'kwargs': kwargs,
            }
        try:
            return self._request(method, uri, **kwargs)
        except exceptions.ZADClientException as e:
            return self._handle_request_except(e, self.request, method, uri, **kwargs)

    def get(self, uri, params=None, **kwargs):
        """
        get 接口请求

        :param uri: 请求url
        :param params: get 参数（dict 格式）
        """
        if params is not None:
            kwargs['params'] = params
        return self.request('GET', uri, **kwargs)

    def post(self, uri, data=None, params=None, **kwargs):
        """
        post 接口请求

        :param uri: 请求url
        :param data: post 数据（dict 格式会自动转换为json）
        :param params: post接口中url问号后参数（dict 格式）
        """
        if data is not None:
            kwargs['data'] = data
        if params is not None:
            kwargs['params'] = params
        return self.request('POST', uri, **kwargs)

    def put(self, uri, data=None, params=None, **kwargs):
        """
        post 接口请求

        :param uri: 请求url
        :param data: post 数据（dict 格式会自动转换为json）
        :param params: put接口中url问号后参数（dict 格式）
        """
        if data is not None:
            kwargs['data'] = data
        if params is not None:
            kwargs['params'] = params
        return self.request('PUT', uri, **kwargs)



# -*- coding: utf-8 -*-


def clean():
    # 页面上直接复制
    text = u"""Report Type	Metric	Tactic	Description
campaigns 	campaignName 	T00020,T00030,remarketing 	The name of the campaign.
campaigns 	campaignId 	T00020,T00030,remarketing 	The identifier of the campaign.
campaigns 	impressions 	T00020,T00030,remarketing 	Total number of ad impressions.
campaigns 	clicks 	T00020,T00030,remarketing 	Total number of ad clicks associated with the campaign.
campaigns 	cost 	T00020,T00030,remarketing 	The total cost of all ad clicks for the campaign. Divide cost by clicks to calculate average cost per click (CPC).
campaigns 	currency 	T00020,T00030,remarketing 	The currency code associated with the campaign.
campaigns 	attributedConversions1d 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 24 hours of ad click.
campaigns 	attributedConversions7d 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 7 days of ad click.
campaigns 	attributedConversions14d 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 14 days of ad click.
campaigns 	attributedConversions30d 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 30 days of ad click.
campaigns 	attributedConversions1dSameSKU 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 24 hours of ad click, where the SKU of the product advertised and the SKU of the conversion event are equivalent.
campaigns 	attributedConversions7dSameSKU 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 7 days of ad click, where the SKU of the product advertised and the SKU of the conversion event are equivalent.
campaigns 	attributedConversions14dSameSKU 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 14 days of ad click, where the SKU of the product advertised and the SKU of the conversion event are equivalent.
campaigns 	attributedConversions30dSameSKU 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 30 days of ad click, where the SKU of the product advertised and the SKU of the conversion event are equivalent.
campaigns 	attributedUnitsOrdered1d 	T00020,T00030,remarketing 	Total number of attributed units ordered within 24 hours of ad click.
campaigns 	attributedUnitsOrdered7d 	T00020,T00030,remarketing 	Total number of attributed units ordered within 7 days of ad click.
campaigns 	attributedUnitsOrdered14d 	T00020,T00030,remarketing 	Total number of attributed units ordered within 14 days of ad click.
campaigns 	attributedUnitsOrdered30d 	T00020,T00030,remarketing 	Total number of attributed units ordered within 30 days of ad click.
campaigns 	attributedSales1d 	T00020,T00030,remarketing 	Total number of attributed sales occurring within 24 hours of ad click.
campaigns 	attributedSales7d 	T00020,T00030,remarketing 	Total number of attributed sales occurring within 7 days of ad click.
campaigns 	attributedSales14d 	T00020,T00030,remarketing 	Total number of attributed sales occurring within 14 days of ad click.
campaigns 	attributedSales30d 	T00020,T00030,remarketing 	Total number of attributed sales occurring within 30 days of ad click.
campaigns 	attributedSales1dSameSKU 	T00020,T00030,remarketing 	Aggregate value of all attributed sales occurring within 24 hours of ad click, where the SKU of the product advertised and the SKU of the purchased item are equivalent.
campaigns 	attributedSales7dSameSKU 	T00020,T00030,remarketing 	Aggregate value of all attributed sales occurring within 7 days of ad click, where the SKU of the product advertised and the SKU of the purchased item are equivalent.
campaigns 	attributedSales14dSameSKU 	T00020,T00030,remarketing 	Aggregate value of all attributed sales occurring within 14 days of ad click, where the SKU of the product advertised and the SKU of the purchased item are equivalent.
campaigns 	attributedSales30dSameSKU 	T00020,T00030,remarketing 	Aggregate value of all attributed sales occurring within 30 days of ad click, where the SKU of the product advertised and the SKU of the purchased item are equivalent.
campaigns 	attributedOrdersNewToBrand14d 	T00020,T00030 	The number of first-time orders for products within the brand over a one-year lookback window.
campaigns 	attributedSalesNewToBrand14d 	T00020,T00030 	The total sales of new-to-brand orders for products within the brand over a one-year lookback window.
campaigns 	attributedUnitsOrderedNewToBrand14d 	T00020,T00030 	The number of units from first-time orders for products within the brand over a one-year lookback window.
campaigns 	costType 	T00020,T00030 	[NEW] Determines how the campaign will bid and charge.
campaigns 	viewImpressions 	T00020,T00030 	[NEW for 'vCPM' campaigns] Total number of ad viewable impressions.
campaigns 	viewAttributedConversions14d 	T00020,T00030 	[NEW for 'vCPM' campaigns] Total number of attributed conversion events occurring within 14 days of ad click or view.
campaigns 	viewAttributedSales14d 	T00020,T00030 	[NEW for 'vCPM' campaigns] Total number of attributed sales occurring within 14 days of ad click or view.
campaigns 	viewAttributedUnitsOrdered14d 	T00020,T00030 	[NEW for 'vCPM' campaigns] Total number of attributed units ordered occurring within 14 days of ad click or view.
adGroups 	campaignName 	T00020,T00030,remarketing 	The name of the campaign.
adGroups 	campaignId 	T00020,T00030,remarketing 	The identifier of the campaign.
adGroups 	adGroupName 	T00020,T00030,remarketing 	The name of the ad group.
adGroups 	adGroupId 	T00020,T00030,remarketing 	The identifier of the ad group.
adGroups 	impressions 	T00020,T00030,remarketing 	Total number of ad impressions.
adGroups 	clicks 	T00020,T00030,remarketing 	Total number of ad clicks associated with the campaign.
adGroups 	cost 	T00020,T00030,remarketing 	The total cost of all ad clicks for the campaign. Divide cost by clicks to calculate average cost per click (CPC).
adGroups 	currency 	T00020,T00030,remarketing 	The currency code associated with the campaign.
adGroups 	attributedConversions1d 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 24 hours of ad click.
adGroups 	attributedConversions7d 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 7 days of ad click.
adGroups 	attributedConversions14d 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 14 days of ad click.
adGroups 	attributedConversions30d 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 30 days of ad click.
adGroups 	attributedConversions1dSameSKU 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 24 hours of ad click, where the SKU of the product advertised and the SKU of the conversion event are equivalent.
adGroups 	attributedConversions7dSameSKU 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 7 days of ad click, where the SKU of the product advertised and the SKU of the conversion event are equivalent.
adGroups 	attributedConversions14dSameSKU 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 14 days of ad click, where the SKU of the product advertised and the SKU of the conversion event are equivalent.
adGroups 	attributedConversions30dSameSKU 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 30 days of ad click, where the SKU of the product advertised and the SKU of the conversion event are equivalent.
adGroups 	attributedUnitsOrdered1d 	T00020,T00030,remarketing 	Total number of attributed units ordered within 24 hours of ad click.
adGroups 	attributedUnitsOrdered7d 	T00020,T00030,remarketing 	Total number of attributed units ordered within 7 days of ad click.
adGroups 	attributedUnitsOrdered14d 	T00020,T00030,remarketing 	Total number of attributed units ordered within 14 days of ad click.
adGroups 	attributedUnitsOrdered30d 	T00020,T00030,remarketing 	Total number of attributed units ordered within 30 days of ad click.
adGroups 	attributedSales1d 	T00020,T00030,remarketing 	Total number of attributed sales occurring within 24 hours of ad click.
adGroups 	attributedSales7d 	T00020,T00030,remarketing 	Total number of attributed sales occurring within 7 days of ad click.
adGroups 	attributedSales14d 	T00020,T00030,remarketing 	Total number of attributed sales occurring within 14 days of ad click.
adGroups 	attributedSales30d 	T00020,T00030,remarketing 	Total number of attributed sales occurring within 30 days of ad click.
adGroups 	attributedSales1dSameSKU 	T00020,T00030,remarketing 	Aggregate value of all attributed sales occurring within 24 hours of ad click, where the SKU of the product advertised and the SKU of the purchased item are equivalent.
adGroups 	attributedSales7dSameSKU 	T00020,T00030,remarketing 	Aggregate value of all attributed sales occurring within 7 days of ad click, where the SKU of the product advertised and the SKU of the purchased item are equivalent.
adGroups 	attributedSales14dSameSKU 	T00020,T00030,remarketing 	Aggregate value of all attributed sales occurring within 14 days of ad click, where the SKU of the product advertised and the SKU of the purchased item are equivalent.
adGroups 	attributedSales30dSameSKU 	T00020,T00030,remarketing 	Aggregate value of all attributed sales occurring within 30 days of ad click, where the SKU of the product advertised and the SKU of the purchased item are equivalent.
adGroups 	attributedOrdersNewToBrand14d 	T00020,T00030 	The number of first-time orders for products within the brand over a one-year lookback window.
adGroups 	attributedSalesNewToBrand14d 	T00020,T00030 	The total sales of new-to-brand orders for products within the brand over a one-year lookback window.
adGroups 	attributedUnitsOrderedNewToBrand14d 	T00020,T00030 	The number of units from first-time orders for products within the brand over a one-year lookback window.
adGroups 	bidOptimization 	T00020,T00030 	[NEW for 'vCPM' campaigns] Bid Optimization for the Adgroup. Default behavior is to optimize for 'clicks'.
adGroups 	viewImpressions 	T00020,T00030 	[NEW for 'vCPM' campaigns] Total number of ad viewable impressions.
adGroups 	viewAttributedConversions14d 	T00020,T00030 	[NEW for 'vCPM' campaigns] Total number of attributed conversion events occurring within 14 days of ad click or view.
adGroups 	viewAttributedSales14d 	T00020,T00030 	[NEW for 'vCPM' campaigns] Total number of attributed sales occurring within 14 days of ad click or view.
adGroups 	viewAttributedUnitsOrdered14d 	T00020,T00030 	[NEW for 'vCPM' campaigns] Total number of attributed units ordered occurring within 14 days of ad click or view.
productAds 	campaignName 	T00020,T00030,remarketing 	The name of the campaign.
productAds 	campaignId 	T00020,T00030,remarketing 	The identifier of the campaign.
productAds 	adGroupName 	T00020,T00030,remarketing 	The name of the ad group.
productAds 	adGroupId 	T00020,T00030,remarketing 	The identifier of the ad group.
productAds 	asin 	T00020,T00030,remarketing 	The ASIN of the product.
productAds 	sku 	T00020,T00030,remarketing 	The SKU of the product.
productAds 	adId 	T00020,T00030,remarketing 	The unique numerical ID of the ad.
productAds 	impressions 	T00020,T00030,remarketing 	Total number of ad impressions.
productAds 	clicks 	T00020,T00030,remarketing 	Total number of ad clicks associated with the campaign.
productAds 	cost 	T00020,T00030,remarketing 	The total cost of all ad clicks for the campaign. Divide cost by clicks to calculate average cost per click (CPC).
productAds 	currency 	T00020,T00030,remarketing 	The currency code associated with the campaign.
productAds 	attributedConversions1d 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 24 hours of ad click.
productAds 	attributedConversions7d 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 7 days of ad click.
productAds 	attributedConversions14d 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 14 days of ad click.
productAds 	attributedConversions30d 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 30 days of ad click.
productAds 	attributedConversions1dSameSKU 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 24 hours of ad click, where the SKU of the product advertised and the SKU of the conversion event are equivalent.
productAds 	attributedConversions7dSameSKU 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 7 days of ad click, where the SKU of the product advertised and the SKU of the conversion event are equivalent.
productAds 	attributedConversions14dSameSKU 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 14 days of ad click, where the SKU of the product advertised and the SKU of the conversion event are equivalent.
productAds 	attributedConversions30dSameSKU 	T00020,T00030,remarketing 	Total number of attributed conversion events occurring within 30 days of ad click, where the SKU of the product advertised and the SKU of the conversion event are equivalent.
productAds 	attributedUnitsOrdered1d 	T00020,T00030,remarketing 	Total number of attributed units ordered within 24 hours of ad click.
productAds 	attributedUnitsOrdered7d 	T00020,T00030,remarketing 	Total number of attributed units ordered within 7 days of ad click.
productAds 	attributedUnitsOrdered14d 	T00020,T00030,remarketing 	Total number of attributed units ordered within 14 days of ad click.
productAds 	attributedUnitsOrdered30d 	T00020,T00030,remarketing 	Total number of attributed units ordered within 30 days of ad click.
productAds 	attributedSales1d 	T00020,T00030,remarketing 	Total number of attributed sales occurring within 24 hours of ad click.
productAds 	attributedSales7d 	T00020,T00030,remarketing 	Total number of attributed sales occurring within 7 days of ad click.
productAds 	attributedSales14d 	T00020,T00030,remarketing 	Total number of attributed sales occurring within 14 days of ad click.
productAds 	attributedSales30d 	T00020,T00030,remarketing 	Total number of attributed sales occurring within 30 days of ad click.
productAds 	attributedSales1dSameSKU 	T00020,T00030,remarketing 	Aggregate value of all attributed sales occurring within 24 hours of ad click, where the SKU of the product advertised and the SKU of the purchased item are equivalent.
productAds 	attributedSales7dSameSKU 	T00020,T00030,remarketing 	Aggregate value of all attributed sales occurring within 7 days of ad click, where the SKU of the product advertised and the SKU of the purchased item are equivalent.
productAds 	attributedSales14dSameSKU 	T00020,T00030,remarketing 	Aggregate value of all attributed sales occurring within 14 days of ad click, where the SKU of the product advertised and the SKU of the purchased item are equivalent.
productAds 	attributedSales30dSameSKU 	T00020,T00030,remarketing 	Aggregate value of all attributed sales occurring within 30 days of ad click, where the SKU of the product advertised and the SKU of the purchased item are equivalent.
productAds 	attributedOrdersNewToBrand14d 	T00020,T00030 	The number of first-time orders for products within the brand over a one-year lookback window.
productAds 	attributedSalesNewToBrand14d 	T00020,T00030 	The total sales of new-to-brand orders for products within the brand over a one-year lookback window.
productAds 	attributedUnitsOrderedNewToBrand14d 	T00020,T00030 	The number of units from first-time orders for products within the brand over a one-year lookback window.
productAds 	viewImpressions 	T00020,T00030 	[NEW for 'vCPM' campaigns] Total number of ad viewable impressions.
productAds 	viewAttributedConversions14d 	T00020,T00030 	[NEW for 'vCPM' campaigns] Total number of attributed conversion events occurring within 14 days of ad click or view.
productAds 	viewAttributedSales14d 	T00020,T00030 	[NEW for 'vCPM' campaigns] Total number of attributed sales occurring within 14 days of ad click or view.
productAds 	viewAttributedUnitsOrdered14d 	T00020,T00030 	[NEW for 'vCPM' campaigns] Total number of attributed units ordered occurring within 14 days of ad click or view.
targets 	campaignName 	T00020,T00030 	The name of the campaign.
targets 	campaignId 	T00020,T00030 	The identifier of the campaign.
targets 	adGroupName 	T00020,T00030 	The name of the ad group.
targets 	adGroupId 	T00020,T00030 	The identifier of the ad group.
targets 	targetId 	T00020,T00030 	The identifier of the targeting expression used in a bid.
targets 	targetingExpression 	T00020,T00030 	The string representation of your expression object in targeting clauses.
targets 	targetingText 	T00020,T00030 	The resolved string representation of your expression object in targeting clauses.
targets 	targetingType 	T00020,T00030 	The type of match for the targeting expression used in bid. For manually created expressions, this value is TARGETING_EXPRESSION. For auto-targeting expressions this value is TARGETING_EXPRESSION_PREDEFINED.
targets 	impressions 	T00020,T00030 	Total number of ad impressions.
targets 	clicks 	T00020,T00030 	Total number of ad clicks associated with the campaign.
targets 	cost 	T00020,T00030 	The total cost of all ad clicks for the campaign. Divide cost by clicks to calculate average cost per click (CPC).
targets 	currency 	T00020,T00030 	The currency code associated with the campaign.
targets 	attributedConversions1d 	T00020,T00030 	Total number of attributed conversion events occurring within 24 hours of ad click.
targets 	attributedConversions7d 	T00020,T00030 	Total number of attributed conversion events occurring within 7 days of ad click.
targets 	attributedConversions14d 	T00020,T00030 	Total number of attributed conversion events occurring within 14 days of ad click.
targets 	attributedConversions30d 	T00020,T00030 	Total number of attributed conversion events occurring within 30 days of ad click.
targets 	attributedConversions1dSameSKU 	T00020,T00030 	Total number of attributed conversion events occurring within 24 hours of ad click, where the SKU of the product advertised and the SKU of the conversion event are equivalent.
targets 	attributedConversions7dSameSKU 	T00020,T00030 	Total number of attributed conversion events occurring within 7 days of ad click, where the SKU of the product advertised and the SKU of the conversion event are equivalent.
targets 	attributedConversions14dSameSKU 	T00020,T00030 	Total number of attributed conversion events occurring within 14 days of ad click, where the SKU of the product advertised and the SKU of the conversion event are equivalent.
targets 	attributedConversions30dSameSKU 	T00020,T00030 	Total number of attributed conversion events occurring within 30 days of ad click, where the SKU of the product advertised and the SKU of the conversion event are equivalent.
targets 	attributedUnitsOrdered1d 	T00020,T00030 	Total number of attributed units ordered within 24 hours of ad click.
targets 	attributedUnitsOrdered7d 	T00020,T00030 	Total number of attributed units ordered within 7 days of ad click.
targets 	attributedUnitsOrdered14d 	T00020,T00030 	Total number of attributed units ordered within 14 days of ad click.
targets 	attributedUnitsOrdered30d 	T00020,T00030 	Total number of attributed units ordered within 30 days of ad click.
targets 	attributedSales1d 	T00020,T00030 	Total number of attributed sales occurring within 24 hours of ad click.
targets 	attributedSales7d 	T00020,T00030 	Total number of attributed sales occurring within 7 days of ad click.
targets 	attributedSales14d 	T00020,T00030 	Total number of attributed sales occurring within 14 days of ad click.
targets 	attributedSales30d 	T00020,T00030 	Total number of attributed sales occurring within 30 days of ad click.
targets 	attributedSales1dSameSKU 	T00020,T00030 	Aggregate value of all attributed sales occurring within 24 hours of ad click, where the SKU of the product advertised and the SKU of the purchased item are equivalent.
targets 	attributedSales7dSameSKU 	T00020,T00030 	Aggregate value of all attributed sales occurring within 7 days of ad click, where the SKU of the product advertised and the SKU of the purchased item are equivalent.
targets 	attributedSales14dSameSKU 	T00020,T00030 	Aggregate value of all attributed sales occurring within 14 days of ad click, where the SKU of the product advertised and the SKU of the purchased item are equivalent.
targets 	attributedSales30dSameSKU 	T00020,T00030 	Aggregate value of all attributed sales occurring within 30 days of ad click, where the SKU of the product advertised and the SKU of the purchased item are equivalent.
targets 	attributedOrdersNewToBrand14d 	T00020,T00030 	The number of first-time orders for products within the brand over a one-year lookback window.
targets 	attributedSalesNewToBrand14d 	T00020,T00030 	The total sales of new-to-brand orders for products within the brand over a one-year lookback window.
targets 	attributedUnitsOrderedNewToBrand14d 	T00020,T00030 	The number of units from first-time orders for products within the brand over a one-year lookback window.
targets 	viewImpressions 	T00020,T00030 	[NEW for 'vCPM' campaigns] Total number of ad viewable impressions.
targets 	viewAttributedConversions14d 	T00020,T00030 	[NEW for 'vCPM' campaigns] Total number of attributed conversion events occurring within 14 days of ad click or view.
targets 	viewAttributedSales14d 	T00020,T00030 	[NEW for 'vCPM' campaigns] Total number of attributed sales occurring within 14 days of ad click or view.
targets 	viewAttributedUnitsOrdered14d 	T00020,T00030 	[NEW for 'vCPM' campaigns] Total number of attributed units ordered occurring within 14 days of ad click or view.
asins 	campaignName 	T00020,T00030 	The name of the campaign.
asins 	campaignId 	T00020,T00030 	The identifier of the campaign.
asins 	adGroupName 	T00020,T00030 	The name of the ad group.
asins 	adGroupId 	T00020,T00030 	The identifier of the ad group.
asins 	asin 	T00020,T00030 	The ASIN of the product.
asins 	otherAsin 	T00020,T00030 	The ASIN of the product other than the one advertised
asins 	sku 	T00020,T00030 	The SKU of the product.
asins 	currency 	T00020,T00030 	The currency code associated with the campaign.
asins 	attributedUnitsOrdered1dOtherSKU 	T00020,T00030 	Number of other ASIN (SKU) units sold. 1 day.
asins 	attributedUnitsOrdered7dOtherSKU 	T00020,T00030 	Number of other ASIN (SKU) units sold. 7 days.
asins 	attributedUnitsOrdered14dOtherSKU 	T00020,T00030 	Number of other ASIN (SKU) units sold. 14 days.
asins 	attributedUnitsOrdered30dOtherSKU 	T00020,T00030 	Number of other ASIN (SKU) units sold. 30 days.
asins 	attributedSales1dOtherSKU 	T00020,T00030 	Sales for another ASIN (SKU). 1 day.
asins 	attributedSales7dOtherSKU 	T00020,T00030 	Sales for another ASIN (SKU). 7 days.
asins 	attributedSales14dOtherSKU 	T00020,T00030 	Sales for another ASIN (sku). 14 days.
asins 	attributedSales30dOtherSKU 	T00020,T00030 	Sales for another ASIN (sku). 30 days.
campaigns 	campaignName 	T00001 	The name of the campaign.
campaigns 	campaignId 	T00001 	The identifier of the campaign.
campaigns 	campaignStatus 	T00001 	The status of the campaign.
campaigns 	currency 	T00001 	The currency code associated with the campaign.
campaigns 	impressions 	T00001 	Total number of ad impressions.
campaigns 	clicks 	T00001 	Total number of ad clicks associated with the campaign. Divide clicks by impressions to calculate click through rate (CTR).
campaigns 	cost 	T00001 	The total cost of all ad clicks for the campaign. Divide cost by clicks to calculate average cost per click (CPC).
campaigns 	attributedDPV14d 	T00001 	Number of attributed detail page views occurring within 14 days of click on an ad.
campaigns 	attributedUnitsSold14d 	T00001 	Number of attributed units sold occurring within 14 days of click on an ad.
campaigns 	attributedSales14d 	T00001 	Aggregate value of attributed sales occurring within 14 days of click on ad.
"""

    text_list = [i.strip() for i in text.split('\n') if i.strip()]

    keys = [i.strip().replace(' ', '_').lower() for i in text_list[0].split('\t') if i.strip()]
    keys_len = len(keys)

    result = dict()

    others = text_list[1:]
    for index, row in enumerate(others):
        row = [i.strip() for i in row.split('\t') if i.strip()]
        assert len(row) == keys_len
        # print(row)

        tmp = dict(zip(keys, row))
        # print(tmp)

        tactic = tmp['tactic']
        if ',' in tactic:
            tactic = [i.strip() for i in tactic.split(',') if i.strip()]
        else:
            tactic = [tactic]
        tmp['tactic'] = tactic

        print(tmp)
        # {'report_type': 'campaigns', 'metric': 'campaignName', 'tactic': ['T00020', 'T00030', 'remarketing'], 'description': 'The name of the campaign.'}

        report_type = tmp['report_type']
        metric = tmp['metric']
        tactic_list = tmp['tactic']
        description = tmp['description']

        for tactic in tactic_list:
            try:
                result[report_type][tactic].append(metric)
            except KeyError:
                if report_type not in result:
                    result[report_type] = dict()
                    result[report_type][tactic] = [metric]
                else:
                    if tactic not in result[report_type]:
                        result[report_type][tactic] = [metric]
        print(result)
        print('----------------------------------------')

        if index == 200000000:
            break

    return result


if __name__ == '__main__':
    res = clean()
    print(res)



# -*- coding: utf-8 -*-
# Authored by: Josh (joshzda@gmail.com)
import random
import string

import six


def to_text(value, encoding='utf-8'):
    """Convert value to unicode, default encoding is utf-8

    :param value: Value to be converted
    :param encoding: Desired encoding
    """
    if not value:
        return ''
    if isinstance(value, six.text_type):
        return value
    if isinstance(value, six.binary_type):
        return value.decode(encoding)
    return six.text_type(value)


def to_binary(value, encoding='utf-8'):
    """Convert value to binary string, default encoding is utf-8

    :param value: Value to be converted
    :param encoding: Desired encoding
    """
    if not value:
        return b''
    if isinstance(value, six.binary_type):
        return value
    if isinstance(value, six.text_type):
        return value.encode(encoding)
    return to_text(value).encode(encoding)


def random_string(length=16):
    rule = string.ascii_letters + string.digits
    rand_list = random.sample(rule, length)
    return ''.join(rand_list)


def byte2int(c):
    if six.PY2:
        return ord(c)
    return c


# -*- coding: utf-8 -*-
# Authored by: Josh (joshzda@gmail.com)
from setuptools import setup

setup(
    name='amazon_ad_sdk',
    version='0.0.1b32',
    packages=['amazon_ad',
              'amazon_ad.api',
              'amazon_ad.api.sb',
              'amazon_ad.api.sp',
              'amazon_ad.api.sd',
              'amazon_ad.api.dsp',
              'amazon_ad.client',
              'amazon_ad.core',
              'amazon_ad.core.utils',
              ],
    url='',
    license='',
    author='Ryan',
    author_email='931798845@qq.com',
    description='Amazon Advertising SDK',
    setup_requires=['wheel'],
    install_requires=[
        "arrow==0.12.1",
        "certifi==2018.10.15",
        "chardet==3.0.4",
        "idna==2.7",
        "ipython==7.2.0",
        "python-dateutil==2.7.5",
        "requests==2.20.1",
        "six==1.11.0",
        "urllib3==1.24.1",
        "wcwidth==0.1.9",
    ],
    python_requires='>=3.6',
)



# -*- coding: utf-8 -*-
# Authored by: Josh (joshzda@gmail.com)


class ZADBaseAPI(object):
    API_BASE_URL = None

    def __init__(self, client=None):
        self._client = client

    def _get(self, url, **kwargs):
        if self.API_BASE_URL:
            kwargs['api_base_url'] = self.API_BASE_URL
        return self._client.get(url, **kwargs)

    def _post(self, url, **kwargs):
        if self.API_BASE_URL:
            kwargs['api_base_url'] = self.API_BASE_URL
        return self._client.post(url, **kwargs)

    def _put(self, url, **kwargs):
        if self.API_BASE_URL:
            kwargs['api_base_url'] = self.API_BASE_URL
        return self._client.put(url, **kwargs)


class ZADAPI(ZADBaseAPI):

    def get_header(self):
        return {}

    def _get(self, url, params=None, **kwargs):
        kwargs['params'] = params
        kwargs['headers'] = self.get_header()
        return super(ZADAPI, self)._get(url, **kwargs)

    def _post(self, url, params=None, data=None, **kwargs):
        kwargs['params'] = params
        kwargs['data'] = data
        kwargs['headers'] = self.get_header()
        return super(ZADAPI, self)._post(url, **kwargs)

    def _put(self, url, params=None, data=None, **kwargs):
        kwargs['params'] = params
        kwargs['data'] = data
        kwargs['headers'] = self.get_header()
        return super(ZADAPI, self)._put(url, **kwargs)

    def get(self, url, params=None, **kwargs):
        return self._get(url, params, **kwargs)

    def post(self, url, data=None, params=None, **kwargs):
        return self._post(url, params, data, **kwargs)

    def put(self, url, data=None, params=None, **kwargs):
        return self._put(url, params, data, **kwargs)

    def download(self, url, params=None, **kwargs):
        kwargs['stream'] = True
        return self._get(url, params, **kwargs)



class ZADAuthAPI(ZADAPI):

    def get_header(self):
        _headers = {
            "Content-Type": "application/x-www-form-urlencoded",
            "charset": "UTF-8",
            "Connection": "close"
        }

        return _headers


class ZADOpenAPI(ZADAPI):

    def get_header(self):
        _headers = {
            'Amazon-Advertising-API-ClientId': self._client.client_id,
            'Authorization': 'Bearer %s' % self._client.access_token,
            'Content-Type': 'application/json',
            'Connection': 'close',
        }

        if self._client.profile_id is not None:
            _headers.update(
                {
                    "Amazon-Advertising-API-Scope": self._client.profile_id
                }
            )

        return _headers
